
/**
 * Hlavná trieda projektu. V tomto projekte definujeme v tejto triede
 * všetko podstatné týkajúce sa údajov aj správania hry.
 */
public class HlavnáTrieda extends Robot
{
	// Zoznam robotov reprezentujúcich hlavu a telo hadíka:
	private final Zoznam<Robot> hadík = new Zoznam<Robot>();

	/**
	 * Súkromný konštruktor (dôvody som vysvetľoval na prednáškach). V ňom
	 * incializujeme všetko potrebné a definujeme obsluhu udalostí.
	 */
	private HlavnáTrieda()
	{
		// Skrytie hlavného robota (nebudeme ho potrebovať):
		skry();

		// Keď chceme premennú používať v obsluhe udalostí, musí byť
		// definovaná ako finálna (konečná = final)… To sa bude týkať
		// hlavy hadíka a jedla.

		// Vytvoríme hlavu. Bude to „obyčajný“ robot, chcem povedať, že pre
		// hlavu sme nedefinovali žiadnu samostatnú triedu ako pre články
		// tela a jedlo.
		final Robot hlava = new Robot();

		// Nastavíme hlave príslušné vlastnosti:
		hlava.farba(oranžová);
		hlava.zdvihniPero();
		hlava.veľkosť(15);

		// Pridáme hlavu do zoznamu, ktorý je na tomto mieste prázdny, čiže
		// hlava bude prvým prvkom zoznamu.
		hadík.pridaj(hlava);

		// Vytvoríme jedlo. Všetky vlastnosti sú nastavené v konštruktore
		// triedy Jedlo.
		final Robot jedlo = new Jedlo();

		// Vytvoríme 9 článkov tela a pridáme ich do zoznamu. Všetky
		// vlastnosti článkov sú nastavované v konštruktore triedy
		// ČlánokTela.
		for (int i = 0; i < 9; ++i)
			hadík.pridaj(new ČlánokTela(i));

		// Definujeme obsluhu udalostí:
		new ObsluhaUdalostí()
		{
			// Tik bude reagovať na časovač.
			@Override public void tik()
			{
				// V cykle postupne posunieme všetky články tela na pozíciu
				// predchádzajúceho článku v zozname. Tým zabezpečíme
				// postupné posúvanie celého tela hadíka pri každom tiku.
				for (int i = hadík.veľkosť() - 1; i > 0; --i)
				{
					hadík.daj(i).skočNa(hadík.daj(i - 1));
					hadík.daj(i).otoč(hadík.daj(i - 1));
				}

				// Keď chcete, aby hadík fungoval v automatickom režime,
				// odkomentujte nasledujúce riadky kódu. Podmienka určuje,
				// že s pravdepodobnosťou 20% sa má vykonať príkaz hovoriaci
				// „hlava otoč sa na jedlo“.
				/** /
				if (svet.náhodnéCeléČíslo(0, 100) <= 20)
					hlava.otočNa(jedlo);
				/**/

				// Hlavu posunieme dopredu o 20 bodov.
				hlava.dopredu(20);

				// Nasledovná štvorica podmienených spracovaní zabezpečuje,
				// že ak hlava opustí vidteľnú časť plátna, objaví sa na
				// protiľahlej strane:

				if (hlava.polohaX() > svet.pravýOkraj())
					hlava.polohaX(svet.ľavýOkraj());

				if (hlava.polohaX() < svet.ľavýOkraj())
					hlava.polohaX(svet.pravýOkraj());

				if (hlava.polohaY() > svet.hornýOkraj())
					hlava.polohaY(svet.spodnýOkraj());

				if (hlava.polohaY() < svet.spodnýOkraj())
					hlava.polohaY(svet.hornýOkraj());

				// Nakoniec skontrolujeme, či hlava koliduje s jedlom:
				if (jedlo.koliduje(hlava))
				{
					// Presunutie jedla na novú náhodnú polohu spôsobí
					// zdanie, že na hracej ploche vzniklo nové jedlo…
					jedlo.náhodnáPoloha();
				}

				// V metóde „tik“ musíme prekresľovanie sveta zabezpečovať
				// „ručne“ (tým chcem povedať, že sa nedeje automaticky):
				svet.prekresli();
			}

			// Vďaka tejto metóde môžeme hadíka riadiť klávesnicou:
			@Override public void stlačenieKlávesu()
			{
				switch (údajeUdalostí.kláves())
				{
					case Kláves.vpravo: hlava.smer(0);   break;
					case Kláves.vľavo:  hlava.smer(180); break;
					case Kláves.hore:   hlava.smer(90);  break;
					case Kláves.dole:   hlava.smer(270); break;
				}
			}

			// Vďaka tejto metóde môžeme hadíka riadiť klikmi myšou:
			@Override public void stlačenieTlačidlaMyši()
			{
				hlava.otočNaMyš();
			}

			// Vďaka tejto metóde môžeme hadíka riadiť ťahaním myši:
			@Override public void ťahanieMyšou()
			{
				hlava.otočNaMyš();
			}
		};

		// Po dokončení všetkých príkazov konštruktora spustíme časovač:
		svet.spustiČasovač(0.05);
	}

	/**
	 * Hlavná metóda. Spomíname ju od začiatku semestra, takže jej účel by
	 * mal byť zrejmý…
	 */
	public static void main(String[] args)
	{
		new HlavnáTrieda();
	}
}
